package com.lc.Runner;

import org.junit.runner.Description;
import org.junit.runner.Runner;
import org.junit.runner.notification.Failure;
import org.junit.runner.notification.RunNotifier;
import redis.clients.jedis.JedisPool;
import redis.clients.jedis.JedisPoolConfig;

/**
 * 测试用例
 */
public class SpringJunitRunner extends Runner {


    private Class<?> testClass;
    private JedisPool jedisPool;

    public SpringJunitRunner(Class<?> testClass) {
        this.testClass = testClass;
    }


    @Override
    public Description getDescription() {
        return Description.createTestDescription(testClass, "Redis Pool Test");
    }

    @Override
    public void run(RunNotifier runNotifier) {
        // 初始化Jedis连接池
        JedisPoolConfig config = new JedisPoolConfig();
        config.setMaxTotal(10);
        config.setMaxIdle(5);
        config.setMinIdle(1);
        jedisPool = new JedisPool(config, "124.221.214.183", 6379);

        // 运行测试用例
        runNotifier.fireTestStarted(getDescription());
        try {
            testClass.newInstance();
        } catch (InstantiationException | IllegalAccessException e) {
            runNotifier.fireTestFailure(new Failure(getDescription(), e));
        }
        runNotifier.fireTestFinished(getDescription());

        // 关闭Jedis连接池
        jedisPool.destroy();
    }

    public JedisPool getJedisPool() {
        return jedisPool;
    }
}
