package com.lc.redisPool;

import redis.clients.jedis.Jedis;

import java.util.Queue;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * redis连接池
 */
public class MyRedisPool implements IRedisPool{

    private int maxTotal;
    public long maxWaitMillis;
    //定义一个空闲的连续集合
    private LinkedBlockingQueue<Jedis> idleObjects=null;
    //定义一个活动的连续集合
    private LinkedBlockingQueue<Jedis> activeObjects=null;
    //总的redis连接数量(int)原子操作粒
    private AtomicInteger count =new AtomicInteger();
    @Override
    public void init(int maxTotal, long maxWaiMillis) {
         this.maxTotal=maxTotal;
         this.maxWaitMillis=maxWaiMillis;
         idleObjects=new LinkedBlockingQueue<Jedis>(maxTotal);
         activeObjects=new LinkedBlockingQueue<Jedis>(maxTotal);
    }

    @Override
    public Jedis getResource() throws Exception {
        //伪代码理思路
        //记录开始时间戳，用于判断超时时间
        long startTime = System.currentTimeMillis();
        //从空闲连接集合中获取连接，如果获取到了一个空闲的连接，将该连接放入活动连接集合中，返回连接
        Jedis redis = null;
        while (redis == null) {
             redis = idleObjects.poll();
            if (redis != null) {
                activeObjects.offer(redis);
                return redis;
            }
            //如果无法拿到连接，判断连接是否已满
            if (count.get() < maxTotal) {
                if (count.incrementAndGet() <= maxTotal) {
                    redis = new Jedis("124.221.214.183", 6379);
                    System.out.println("---------Redis连接池创建了一个连接");
                    activeObjects.offer(redis);
                    return redis;
                } else {
                    count.decrementAndGet();
                }
            }
            try {
                //如果连接满了，等待其他线程释放到空闲连接集合中，如果说一定的空闲时间，可以得到一个空闲连接，
                //将连接放入到活动连接集合，返回该连接
                redis = idleObjects.poll(maxWaitMillis - (System.currentTimeMillis() - startTime), TimeUnit.MILLISECONDS);
                if (null!=redis){
                    activeObjects.offer(redis);
                    return redis;
                }
            }catch (Exception e){
                e.printStackTrace();
            }
            //如果等待时间超过了我们最长的等待时间，抛出等待超时异常
            if (maxWaitMillis < (System.currentTimeMillis() - startTime)){
                throw new Exception("timeout，超过最长等待时间");
            }else {
                continue;
            }
        }
        return redis;
    }

    @Override
    public void release(Jedis jedis) {
        if (activeObjects.remove(jedis)){
            idleObjects.offer(jedis);
        }
    }
}
